@@ -1,4 +1,4 @@
-__version__ = '0.0.9'
+__version__ = '0.0.10'
 
 import threading
 import socket
@@ -45,9 +45,13 @@ def pub_udp(target,
         target: host:port or (host, port) tuple
         topic: topic to publish
         message: message (string, bytes or anyting which can be str())
+
     Optional:
-        need_ack: require server acknowledge (default: True)
-        check_ack_src: check acknowledge source (host/port, default: True)
+        * need_ack: require server acknowledge (default: True)
+        * check_ack_src: check acknowledge source (host/port, default: True)
+        * user: user name
+        * password: password
+        * timeout: socket timeout
     """
     if isinstance(target, str):
         host, port = target.rsplit(':', maxsplit=1)
@@ -199,17 +203,22 @@ class Client:
 
     def _exec_control_command(self, payload, from_pinger=False):
         op_start = time.perf_counter()
-        if not self.control_lock.acquire(timeout=self.timeout):
-            raise TimeoutError(f'client error {self.path} control lock timeout')
         try:
-            self.control_socket.settimeout(
-                reduce_timeout(self.timeout, op_start))
-            self.control_socket.sendall(payload)
-            self.control_socket.settimeout(
-                reduce_timeout(self.timeout, op_start))
-            self._handle_control_response(from_pinger)
-        finally:
-            self.control_lock.release()
+            if not self.control_lock.acquire(timeout=self.timeout):
+                raise TimeoutError(
+                    f'client error {self.path} control lock timeout')
+            try:
+                self.control_socket.settimeout(
+                    reduce_timeout(self.timeout, op_start))
+                self.control_socket.sendall(payload)
+                self.control_socket.settimeout(
+                    reduce_timeout(self.timeout, op_start))
+                self._handle_control_response(from_pinger)
+            finally:
+                self.control_lock.release()
+        except:
+            self._shutdown(from_pinger)
+            raise
 
     def _pinger(self):
         while self.connected:
